from collections import namedtuple
from operator import attrgetter
import calendar
from datetime import datetime, timedelta
from email.utils import parsedate, formatdate

import click
import requests
from cachecontrol import CacheControl
from cachecontrol.caches import FileCache
from cachecontrol.heuristics import BaseHeuristic
from halo import Halo
from selectolax.parser import HTMLParser
from tabulate import tabulate

NEXT_BUTTON_SELECTOR = '#dependents > div.paginate-container > div > a'
ITEM_SELECTOR = '#dependents > div.Box > div.flex-items-center'
REPO_SELECTOR = 'span > a.text-bold'
STARS_SELECTOR = 'div > span:nth-child(1)'
GITHUB_URL = 'https://github.com'


class OneDayHeuristic(BaseHeuristic):

    def update_headers(self, response):
        date = parsedate(response.headers["date"])
        expires = datetime(*date[:6]) + timedelta(days=1)
        return {
            "expires": formatdate(calendar.timegm(expires.timetuple())),
            "cache-control": "public",
        }

    def warning(self, response):
        msg = "Automatically cached! Response is Stale."
        return '110 - "%s"' % msg


@click.command()
@click.argument('url')
@click.option('--show', default=10, help='Number of showing repositories (default=10).')
@click.option('--more-than', default=5, help='Minimum number of stars (default=5).')
def cli(url, show, more_than):
    url = '{}/network/dependents'.format(url)
    Repo = namedtuple('Repo', ['url', 'stars'])
    repos = []
    more_than_zero = 0
    repo_count = 0
    spinner = Halo(text='Fetch information about dependents repo', spinner='dots')
    spinner.start()
    sess = requests.session()
    cached_sess = CacheControl(sess, cache=FileCache(".ghtopdep_cache"), heuristic=OneDayHeuristic())
    while True:
        r = cached_sess.get(url)
        parsed_node = HTMLParser(r.text)
        dependents = parsed_node.css(ITEM_SELECTOR)
        repo_count += len(dependents)
        for i in dependents:
            repo_stars = i.css(STARS_SELECTOR)[0].text().strip()
            repo_stars_num = int(repo_stars.replace(',', ''))
            if repo_stars_num != 0:
                more_than_zero += 1
            if repo_stars_num >= more_than:
                relative_repo_url = i.css(REPO_SELECTOR)[0].attributes['href']
                repo_url = '{}{}'.format(GITHUB_URL, relative_repo_url)
                repos.append(Repo(repo_url, repo_stars_num))

        node = parsed_node.css(NEXT_BUTTON_SELECTOR)
        if len(node) == 2:
            url = node[1].attributes['href']
        elif len(node) == 0 or node[0].text() == "Previous":
            spinner.stop()
            break
        elif node[0].text() == "Next":
            url = node[0].attributes['href']

    sorted_repos = sorted(repos[:show], key=attrgetter('stars'), reverse=True)
    print(tabulate(sorted_repos, headers=['URL', "Stars"], tablefmt="grid"))

    print('found {} repos others repos is private'.format(repo_count))
    print('found {} repos with more than zero star'.format(more_than_zero))



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['ghtopdep']

package_data = \
{'': ['*']}

install_requires = \
['cachecontrol>=0.12.5,<0.13.0',
 'click>=7.0,<8.0',
 'halo>=0.0.26,<0.0.27',
 'lockfile>=0.12.2,<0.13.0',
 'requests>=2.22,<3.0',
 'selectolax>=0.1.13,<0.2.0',
 'tabulate>=0.8.3,<0.9.0']

entry_points = \
{'console_scripts': ['ghtopdep = ghtopdep.ghtopdep:cli']}

setup_kwargs = {
    'name': 'ghtopdep',
    'version': '0.1.3',
    'description': 'CLI tool for sorting dependents repo by stars',
    'long_description': '# GHTOPDEP\n[![image](https://img.shields.io/pypi/v/ghtopdep.svg)](https://pypi.org/project/ghtopdep/)\n[![image](https://img.shields.io/pypi/l/ghtopdep.svg)](https://pypi.org/project/ghtopdep/)\n[![image](https://img.shields.io/pypi/pyversions/ghtopdep.svg)](https://pypi.org/project/ghtopdep/)\n\nCLI tool for sorting dependents repo by stars\n\n## Requirements\n* Python 3.6 and up\n\n## Installation\nfrom PyPI\n```\n$ pip install ghtopdep\n```\n\nfrom git repository\n```\n$ pip install git+https://github.com/github-tooling/ghtopdep\n```\n\nfrom source\n```\n$ git clone https://github.com/github-tooling/ghtopdep\n$ cd ghtopdep\n$ python setup.py install\n```\n\n## Usage\n\n```\n$ ghtopdep --help\nUsage: ghtopdep [OPTIONS] URL\n\nOptions:\n  --show INTEGER       Number of showing repositories (default=10).\n  --more-than INTEGER  Number of stars (default=5).\n  --help               Show this message and exit.\n```\n\n\n```\n$ ghtopdep https://github.com/dropbox/dropbox-sdk-js              \n+-------------------------------------------+---------+\n| URL                                       |   Stars |\n+===========================================+=========+\n| https://github.com/LN-Zap/zap-desktop     |     979 |\n+-------------------------------------------+---------+\n| https://github.com/Cleod9/syncmarx-webext |      35 |\n+-------------------------------------------+---------+\n| https://github.com/Playork/StickyNotes    |      32 |\n+-------------------------------------------+---------+\n| https://github.com/WebAssemblyOS/wasmos   |      23 |\n+-------------------------------------------+---------+\n| https://github.com/Cleod9/syncmarx-api    |      19 |\n+-------------------------------------------+---------+\n| https://github.com/Bearer/templates       |      11 |\n+-------------------------------------------+---------+\n| https://github.com/nathsimpson/isobel     |       9 |\n+-------------------------------------------+---------+\n| https://github.com/sorentycho/blackmirror |       8 |\n+-------------------------------------------+---------+\n| https://github.com/easylogic/edy          |       6 |\n+-------------------------------------------+---------+\n| https://github.com/Kikugumo/imas765probot |       5 |\n+-------------------------------------------+---------+\nfound 1573 repos others repos is private\nfound 417 repos with more than zero star\n```\n\n\n## Development setup\nUsing [Poetry](https://poetry.eustace.io/docs/)   \n```\n$ poetry install\n```\nor [Pipenv](https://docs.pipenv.org/)   \n```\n$ pipenv install --dev -e .\n```\n\n## License\n[MIT](https://choosealicense.com/licenses/mit/)\n',
    'author': 'Andriy Orehov',
    'author_email': 'andriyorehov@gmail.com',
    'url': 'https://github.com/github-tooling/ghtopdep',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)
